/*
 * Developer : Alexey Zakharov (alexey.zakharov@vectioneer.com)
 * All rights reserved. Copyright (c) 2019 VECTIONEER.
 */

#ifndef MOTORCORTEX_ROBOT_CONTROL_CART_TO_JOINT_H
#define MOTORCORTEX_ROBOT_CONTROL_CART_TO_JOINT_H

#include "joint_to_cart.h"
#include "motionSL.pb.h"
#include "pose_ostream.h"
#include <iostream>
#include <mcx-cpp/mcx_status.h>
#include <vector>

namespace mcx_robot_control {

class CartToJoint : public mcx_cpp::McxStatus {

  friend class PoseTransformer;

public:
  CartToJoint() = default;

  explicit CartToJoint(const McxStatus& status) : McxStatus{status} {}

  CartToJoint(const McxStatus& status, const motion_spec_CartToJoint& cart_to_joint)
      : McxStatus{status}, cart_to_joint_{cart_to_joint} {}

  [[nodiscard]] motion_spec_CartPose6 cartPose() const { return cart_to_joint_.cartpose; }

  [[nodiscard]] motion_spec_Twist cartTwist() const { return cart_to_joint_.carttwist; }

  [[nodiscard]] motion_spec_JointPositions jointPose() const { return cart_to_joint_.jointpose; }

  [[nodiscard]] motion_spec_JointVelocities jointVelocity() const { return cart_to_joint_.jointtwist; }

  friend std::ostream& operator<<(std::ostream& os, const CartToJoint& cart_to_joint) {
    os << cart_to_joint.cart_to_joint_.cartpose << ", " << cart_to_joint.cart_to_joint_.carttwist << ", "
       << cart_to_joint.cart_to_joint_.jointpose << ", " << cart_to_joint.cart_to_joint_.jointtwist;

    return os;
  }

private:
  motion_spec_CartToJoint cart_to_joint_{};
};

} // namespace mcx_robot_control

#endif // MOTORCORTEX_ROBOT_CONTROL_CART_TO_JOINT_H
