/*
 * Developer : Alexey Zakharov (alexey.zakharov@vectioneer.com)
 * All rights reserved. Copyright (c) 2019 VECTIONEER.
 */

#ifndef MOTORCORTEX_ROBOT_CONTROL_JOINT_TO_CART_H
#define MOTORCORTEX_ROBOT_CONTROL_JOINT_TO_CART_H

#include "motionSL.pb.h"
#include "pose_ostream.h"
#include <iostream>
#include <mcx-cpp/mcx_status.h>

namespace mcx_robot_control {

class JointToCart : public mcx_cpp::McxStatus {

  friend class PoseTransformer;

public:
  JointToCart() = default;

  explicit JointToCart(const McxStatus& status) : McxStatus{status} {}

  JointToCart(const McxStatus& status, const motion_spec_JointToCart& joint_to_cart)
      : McxStatus{status}, joint_to_cart_{joint_to_cart} {}

  [[nodiscard]] motion_spec_CartPose6 cartPose() const { return joint_to_cart_.cartpose; }

  [[nodiscard]] motion_spec_Twist cartTwist() const { return joint_to_cart_.carttwist; }

  [[nodiscard]] motion_spec_JointPositions jointPose() const { return joint_to_cart_.jointpose; }

  [[nodiscard]] motion_spec_JointVelocities jointVelocity() const { return joint_to_cart_.jointtwist; }

  friend std::ostream& operator<<(std::ostream& os, const JointToCart& joint_to_cart) {
    os << joint_to_cart.joint_to_cart_.jointpose << ", " << joint_to_cart.joint_to_cart_.jointtwist << ", "
       << joint_to_cart.joint_to_cart_.cartpose << ", " << joint_to_cart.joint_to_cart_.carttwist;
    return os;
  }

private:
  motion_spec_JointToCart joint_to_cart_{};
};

} // namespace mcx_robot_control

#endif // MOTORCORTEX_ROBOT_CONTROL_JOINT_TO_CART_H
