/*
 * Developer : Alexey Zakharov (alexey.zakharov@vectioneer.com)
 * All rights reserved. Copyright (c) 2019 VECTIONEER.
 */

#include "movec.h"
#include "utils.h"

namespace mcx_robot_control {

MoveC::MoveC(size_t id, std::vector<Waypoint> waypoints, double angle, double velocity, double acceleration,
             double rotational_velocity, double rotational_acceleration, std::vector<double> ref_joint_coord_rad)
    : id_{id}, waypoints_{std::move(waypoints)}, angle_{angle}, velocity_{velocity}, acceleration_{acceleration},
      rotational_velocity_{rotational_velocity}, rotational_acceleration_{rotational_acceleration},
      ref_joint_coord_rad_{std::move(ref_joint_coord_rad)} {

  // measure message size;
  pb_ostream_t stream = PB_OSTREAM_SIZING;
  size_ = encode(&stream);
}

size_t MoveC::encode(pb_ostream_t* stream) const {
  motion_spec_MoveC move_c{};

  move_c.constraint.type = motion_spec_MOTIONCONSTRAINTTYPE_VELANDACC;
  move_c.constraint.has_velacc_values = true;

  move_c.angle = toDegrees(angle_);
  auto& velacc = move_c.constraint.velacc_values;
  velacc.vMax = velocity_;
  velacc.aMax = acceleration_;
  velacc.has_omegaMax = true;
  velacc.omegaMax = rotational_velocity_;
  velacc.has_alfaMax = true;
  velacc.alfaMax = rotational_acceleration_;

  auto& ref_coord = move_c.referenceJoint;
  for (auto& c : ref_coord.coordinates) {
    if (ref_coord.coordinates_count < ref_joint_coord_rad_.size()) {
      c = ref_joint_coord_rad_[ref_coord.coordinates_count++];
    }
  }

  for (auto& w : move_c.waypoints) {
    if (move_c.waypoints_count < waypoints_.size()) {
      auto& w_in = waypoints_[move_c.waypoints_count++];
      auto& pose = w.pose;
      w.has_segmentVelocity = true;
      w.segmentVelocity = w_in.segmentVelocity();
      for (auto& c : pose.coordinates) {
        if (pose.coordinates_count < w_in.length()) {
          c = w_in.coordinate(pose.coordinates_count++);
        }
      }
    }
  }

  if (pb_encode_submessage(stream, motion_spec_MoveC_fields, &move_c)) {
    return stream->bytes_written;
  }
  return 0;
}

size_t MoveC::size() const { return size_; }

motion_spec_MOTIONTYPE MoveC::type() const { return motion_spec_MOTIONTYPE_ARC; }

size_t MoveC::id() const { return id_; }

} // namespace mcx_robot_control