/*
 * Developer : Alexey Zakharov (alexey.zakharov@vectioneer.com)
 * All rights reserved. Copyright (c) 2019 VECTIONEER.
 */

#include "movej.h"

namespace mcx_robot_control {

MoveJ::MoveJ(size_t id, std::vector<Waypoint> waypoints, double rotational_velocity, double rotational_acceleration)
    : container_{.id = id,
                 .waypoints{std::move(waypoints)},
                 .rotational_velocity = rotational_velocity,
                 .rotational_acceleration = rotational_acceleration} {

  // measure message size;
  pb_ostream_t stream = PB_OSTREAM_SIZING;
  size_ = encode(&stream);
}

bool MoveJ::encodeWaypointsHook(pb_ostream_t* stream, const pb_field_t* field, void* const* arg) {
  const auto container_ptr = static_cast<Container*>(*arg);
  for (auto& w : container_ptr->waypoints) {
    motion_spec_JointWaypoint jw{};
    jw.has_constraint = true;
    jw.constraint = {.type = w.type(), .factor = w.smoothFactor()};
    jw.has_segmentVelocity = true;
    jw.segmentVelocity = w.segmentVelocity();
    size_t i = 0;
    for (auto& coord : jw.pose.coordinates) {
      coord = w.coordinate(i++);
    }
    jw.pose.coordinates_count = static_cast<pb_size_t>(w.length());
    if (!pb_encode_tag_for_field(stream, field)) {
      return false;
    }
    if (!pb_encode_submessage(stream, motion_spec_JointWaypoint_fields, &jw)) {
      return false;
    }
  }
  return true;
}

size_t MoveJ::encode(pb_ostream_t* stream) const {
  motion_spec_MoveJ move_j{};
  Container data{container_};
  auto& constraint = move_j.constraint;
  constraint.type = motion_spec_MOTIONCONSTRAINTTYPE_VELANDACC;
  constraint.has_velacc_values = true;
  auto& velacc = constraint.velacc_values;
  velacc.has_omegaMax = true;
  velacc.omegaMax = container_.rotational_velocity;
  velacc.has_alfaMax = true;
  velacc.alfaMax = container_.rotational_acceleration;
  move_j.waypoints.arg = &data;
  move_j.waypoints.funcs.encode = &MoveJ::encodeWaypointsHook;

  if (pb_encode_submessage(stream, motion_spec_MoveJ_fields, &move_j)) {
    return stream->bytes_written;
  }
  return 0;
}

size_t MoveJ::size() const { return size_; }

motion_spec_MOTIONTYPE MoveJ::type() const { return motion_spec_MOTIONTYPE_JOINTMOTION; }

size_t MoveJ::id() const { return container_.id; }

} // namespace mcx_robot_control
