/*
 * Developer : Alexey Zakharov (alexey.zakharov@vectioneer.com)
 * All rights reserved. Copyright (c) 2019 VECTIONEER.
 */

#include "wait.h"
#include <utility>

namespace mcx_robot_control {

Wait::Wait(size_t id, double time_s, std::string path, bool equal_to)
    : id_{id}, time_s_{time_s}, path_{std::move(path)}, equal_to_{equal_to} {
  // measure message size;
  pb_ostream_t stream = PB_OSTREAM_SIZING;
  size_ = encode(&stream);
}

size_t Wait::encode(pb_ostream_t* stream) const {
  motion_spec_Wait wait{};
  wait.has_timeout = true;
  wait.timeout = time_s_;
  if (!path_.empty()) {
    wait.has_path = true;
    path_.copy(wait.path, sizeof(wait.path));
    wait.has_value = true;
    wait.value = equal_to_;
  }
  if (pb_encode_submessage(stream, motion_spec_Wait_fields, &wait)) {
    return stream->bytes_written;
  }
  return 0;
}

size_t Wait::size() const { return size_; }

motion_spec_MOTIONTYPE Wait::type() const { return motion_spec_MOTIONTYPE_WAIT; }

size_t Wait::id() const { return id_; }

} // namespace mcx_robot_control