/*
 * Developer : Alexey Zakharov (alexey.zakharov@vectioneer.com)
 * All rights reserved. Copyright (c) 2019 VECTIONEER.
 */

#ifndef MOTORCORTEX_CPP_POSE_TRANSFORMER_H
#define MOTORCORTEX_CPP_POSE_TRANSFORMER_H

#include "cart_to_joint.h"
#include "joint_to_cart.h"
#include "utils.h"
#include <mcx-cpp/request.h>
#include <vector>

namespace mcx_robot_control {

typedef std::future<JointToCart> JointToCartReply;
typedef std::future<CartToJoint> CartToJointReply;

class PoseTransformer {
public:
  explicit PoseTransformer(const mcx_cpp::Request& request);

  [[nodiscard]] JointToCartReply calcJointToCartPose(const JointToCart& joint_pose) const;

  [[nodiscard]] JointToCartReply calcJointToCartPose(const std::vector<double>& joint_pose,
                                                     const std::vector<double>& actual_cart_pose) const;

  [[nodiscard]] JointToCartReply calcJointToCartPose(const std::vector<double>& joint_pose) const;

  [[nodiscard]] JointToCartReply calcJointToCartPose(const CartToJoint& joint_pose) const;

  [[nodiscard]] CartToJointReply calcCartToJointPose(const CartToJoint& cart_pose) const;

  [[nodiscard]] CartToJointReply
  calcCartToJointPose(const std::vector<double>& cart_pose, const std::vector<double>& ref_joint_pose,
                      motion_spec_FEATUREFRAMETYPE frame_type = motion_spec_FEATUREFRAMETYPE_TOOLTIP) const;

  [[nodiscard]] CartToJointReply
  calcCartToJointPose(const std::vector<double>& cart_pose,
                      motion_spec_FEATUREFRAMETYPE frame_type = motion_spec_FEATUREFRAMETYPE_TOOLTIP) const;

  [[nodiscard]] CartToJointReply
  calcCartToJointPose(const JointToCart& cart_pose,
                      motion_spec_FEATUREFRAMETYPE frame_type = motion_spec_FEATUREFRAMETYPE_TOOLTIP) const;

private:
  const mcx_cpp::Request& request_;
};

} // namespace mcx_robot_control

#endif // MOTORCORTEX_CPP_POSE_TRANSFORMER_H
