/*
 * Developer : Alexey Zakharov (alexey.zakharov@vectioneer.com)
 * All rights reserved. Copyright (c) 2019 VECTIONEER.
 */

#ifndef MOTORCORTEX_CPP_ROBOT_STATUS_H
#define MOTORCORTEX_CPP_ROBOT_STATUS_H

#include <mcx-cpp/status.h>

namespace mcx_robot_control {

class RobotStatus : public mcx_cpp::Status {
public:
  enum ErrorCode {
    UNKNOWN_ERROR = 0,
    NO_ERROR = 1,
    SET_LOGIC_STATE_TIMEOUT = 2,
    UNKNOWN_STATE_TRANSITION = 3,
    SET_LOGIC_MODE_TIMEOUT = 4,
    UNKNOWN_LOGIC_MODE_TRANSITION = 5,
    SET_INTERPRETER_STATE_TIMEOUT = 6,
    UNKNOWN_INTERPRETER_STATE_TRANSITION = 7,
    MOTION_NOT_ALLOWED = 8,
    NOT_IN_ENGAGE = 9,
    NOT_IN_SEMIAUTO = 10,
    INTERPRETER_NOT_PLAYING = 11,
    ESTOP_ACTIVE = 12,
    MOVE_TIMEOUT = 14
  };

  RobotStatus() = default;

  ~RobotStatus() override = default;

  RobotStatus(mcx_cpp::StatusCode status, ErrorCode error) : status_{status}, error_{error}, info_{errorToStr(error)} {}

  RobotStatus(mcx_cpp::StatusCode status, ErrorCode error, std::string info)
      : status_{status}, error_{error}, info_{std::move(info)} {}

  static std::string errorToStr(ErrorCode error) {
    switch (error) {
    case ErrorCode::NO_ERROR:
      return "All good";
    case ErrorCode::SET_LOGIC_STATE_TIMEOUT:
      return "Timeout during switching to a new state";
    case ErrorCode::UNKNOWN_STATE_TRANSITION:
      return "Unknown transition between states";
    case ErrorCode::SET_LOGIC_MODE_TIMEOUT:
      return "Timeout during switching to a new mode";
    case ErrorCode::UNKNOWN_LOGIC_MODE_TRANSITION:
      return "Unknown transition between modes";
    case ErrorCode::SET_INTERPRETER_STATE_TIMEOUT:
      return "Timeout during switching interpreter to a new state";
    case ErrorCode::UNKNOWN_INTERPRETER_STATE_TRANSITION:
      return "Unknown transition between interpreter states";
    case ErrorCode::MOTION_NOT_ALLOWED:
      return "Motion is not allowed";
    case ErrorCode::NOT_IN_ENGAGE:
      return "System is not in engage";
    case ErrorCode::NOT_IN_SEMIAUTO:
      return "System is not in semi-auto";
    case ErrorCode::INTERPRETER_NOT_PLAYING:
      return "Interpreter is not playing";
    case ErrorCode::ESTOP_ACTIVE:
      return "System is in E-Stop";
    case ErrorCode::MOVE_TIMEOUT:
      return "Timeout during move";
    default:
      return "Unknown error";
    }
  }

  [[nodiscard]] mcx_cpp::StatusCode status() const override { return status_; }

  [[nodiscard]] int error() const override { return static_cast<int>(error_); }

  [[nodiscard]] std::string info() const override { return info_; }

private:
  mcx_cpp::StatusCode status_{mcx_cpp::StatusCode::FAILED};
  ErrorCode error_{ErrorCode::UNKNOWN_ERROR};
  std::string info_{errorToStr(ErrorCode::UNKNOWN_ERROR)};
};

} // namespace mcx_robot_control

#endif // MOTORCORTEX_CPP_ROBOT_STATUS_H
