/*
 * Developer : Alexey Zakharov (alexey.zakharov@vectioneer.com)
 * All rights reserved. Copyright (c) 2019 VECTIONEER.
 */

#ifndef MOTORCORTEX_ROBOT_CONTROL_UTILS_H
#define MOTORCORTEX_ROBOT_CONTROL_UTILS_H

#include <chrono>

namespace mcx_robot_control {

template <typename TimeT = std::chrono::nanoseconds>
struct Measure {
  template <typename F, typename... Args>
  static typename TimeT::rep execution(F&& func, Args&&... args) {
    auto start = std::chrono::steady_clock::now();
    std::forward<decltype(func)>(func)(std::forward<Args>(args)...);
    auto duration = std::chrono::duration_cast<TimeT>(std::chrono::steady_clock::now() - start);
    return duration.count();
  }
};

template <typename F, typename... Args>
double measureSec(F&& func, Args&&... args) {
  return Measure<>::execution(func, args...) / 1e9F;
}

constexpr auto PI = 3.14159265358979323846;

template <typename T>
double toRadians(T degrees) {
  return static_cast<double>(degrees / 180.0 * PI);
}

template <typename T>
std::vector<double> toRadians(const std::initializer_list<T> degrees) {
  std::vector<double> radians;
  radians.reserve(degrees.size());
  for (auto& v : degrees) {
    radians.emplace_back(toRadians(v));
  }
  return radians;
}

template <typename T>
double toDegrees(T radians) {
  return static_cast<double>(radians * 180.0 / PI);
}

template <typename T>
std::vector<double> toDegrees(const std::initializer_list<T>& radians) {
  std::vector<double> degrees;
  degrees.reserve(radians.size());
  for (auto& v : radians) {
    degrees.emplace_back(to_degrees(v));
  }
  return degrees;
}

} // namespace mcx_robot_control

#endif // MOTORCORTEX_ROBOT_CONTROL_UTILS_H
