/*
 * Developer : Alexey Zakharov (alexey.zakharov@vectioneer.com)
 * All rights reserved. Copyright (c) 2019 VECTIONEER.
 */

#include "helper.h"
#include "mcx-robot-control.h"

using namespace std;
using namespace mcx_robot_control;

constexpr double TIMEOUT_SEC = 5000;

void manualCartMode(const RobotCommand& robot) {
  auto cart_done = robot.manualCartMode(TIMEOUT_SEC).get();
  if (cart_done.status() == mcx_cpp::StatusCode::OK) {
    clog << "System is in Manual Cartesian Mode" << endl;
  } else {
    helper::printStatus(cart_done);
    throw(runtime_error(string("Failed to set Manual Cartesian Mode")));
  }
  helper::sleep(3);
}

void manualJointMode(const RobotCommand& robot) {
  auto joint_done = robot.manualJointMode(TIMEOUT_SEC).get();
  if (joint_done.status() == mcx_cpp::StatusCode::OK) {
    clog << "System is in Manual Joint Mode" << endl;
  } else {
    helper::printStatus(joint_done);
    throw(runtime_error(string("Failed to set Manual Joint Mode")));
  }
}

void autoMode(const RobotCommand& robot) {
  auto auto_mode = robot.autoMode(TIMEOUT_SEC).get();
  if (auto_mode.status() == mcx_cpp::StatusCode::OK) {
    clog << "System is in Auto Mode" << endl;
  } else {
    helper::printStatus(auto_mode);
    throw(runtime_error(string("Failed to set Auto Mode")));
  }
}

void semiAutoMode(const RobotCommand& robot) {
  auto semi_auto_mode = robot.semiAutoMode(TIMEOUT_SEC).get();
  if (semi_auto_mode.status() == mcx_cpp::StatusCode::OK) {
    clog << "System is in Semi-auto Mode" << endl;
  } else {
    helper::printStatus(semi_auto_mode);
    throw(runtime_error(string("Failed to set Semi-auto Mode")));
  }
}

int main(int argc, char* argv[]) {

  clog << "Motorcortex Robot Control library version: " << mcx_cpp::version() << endl;

  // setting url
  // url format: ws://address:req_port:sub_port
  std::string url = "wss://localhost:5568:5567";
  mcx_cpp::ConnectionOptions options{"mcx.cert.pem"};

  if (argc > 1) {
    url = std::string(argv[1]);
  }

  // Creating a parameter tree;
  mcx_cpp::ParameterTree parameter_tree;
  mcx_cpp::Request req{parameter_tree};
  mcx_cpp::Subscribe sub{req};
  helper::connect(url, options, parameter_tree, req, sub);

  // Subscribing to the log
  sub.subscribe({"root/logger/logOut"}, "log", 1).get().notify([](auto params) {
    std::string log;
    params[0].value(log);
    clog << "Log: " << log;
  });

  RobotCommand robot{req};

  // acknowledge
  robot.acknowledge();
  // switch on
  robot.engage();

  manualCartMode(robot);
  manualJointMode(robot);
  manualCartMode(robot);
  manualJointMode(robot);
  semiAutoMode(robot);
  manualCartMode(robot);
  autoMode(robot);
  robot.off();
}