/*
 * Developer : Alexey Zakharov (alexey.zakharov@vectioneer.com)
 * All rights reserved. Copyright (c) 2019-2022 VECTIONEER.
 */

#include "helper.h"
#include "mcx-robot-control.h"

using namespace std;
using namespace mcx_robot_control;

int main(int argc, char* argv[]) {

  clog << "Motorcortex Robot Control library version: " << mcx_cpp::version() << endl;

  // setting url
  // url format: ws://address:req_port:sub_port
  std::string url = "wss://localhost:5568:5567";
  //  std::string url = "wss://localhost:5568:5567";
  if (argc > 1) {
    url = std::string(argv[1]);
  }
  mcx_cpp::ConnectionOptions options{"mcx.cert.pem"};

  // Creating a parameter tree;
  mcx_cpp::ParameterTree parameter_tree;
  mcx_cpp::Request req{parameter_tree};
  mcx_cpp::Subscribe sub{req};
  helper::connect(url, options, parameter_tree, req, sub);

  // Subscribing to the log
  sub.subscribe({"root/logger/logOut"}, "log", 1).get().notify([](auto params) {
    std::string log;
    params[0].value(log);
    clog << "Log: " << log;
  });

  RobotCommand robot{req};
  robot.acknowledge();
  robot.engage();
  robot.reset();

  // building motion program
  auto start_position_jnt = toRadians({0.0, 0.0, 90.0, 0.0, 90.0, 0.0});
  auto jpos_1 = toRadians({15.0, 0.0, 90.0, 0.0, 90.0, 0.0});
  auto jpos_2 = toRadians({30.0, 15.0, 90.0, 0.0, 90.0, 0.0});
  auto jpos_3 = toRadians({-30.0, -15.0, 90.0, 15.0, 90.0, 15.0});

  auto moving = robot.moveToPoint(start_position_jnt, 0.1, 1).get();
  if (moving.status() == mcx_cpp::StatusCode::OK) {
    clog << "Moved to point start_position_jnt" << endl;
  } else {
    helper::printStatus(moving);
    throw(runtime_error(string("Failed to moved to point start_position_jnt")));
  }

  moving = robot.moveToPoint(jpos_1, 0.1, 1).get();
  if (moving.status() == mcx_cpp::StatusCode::OK) {
    clog << "Moved to point jpos_1" << endl;
  } else {
    helper::printStatus(moving);
    throw(runtime_error(string("Failed to moved to point jpos_1")));
  }

  moving = robot.moveToPoint(jpos_2, 0.1, 1).get();
  if (moving.status() == mcx_cpp::StatusCode::OK) {
    clog << "Moved to point jpos_2" << endl;
  } else {
    helper::printStatus(moving);
    throw(runtime_error(string("Failed to moved to point jpos_2")));
  }

  moving = robot.moveToPoint(jpos_3, 0.1, 1).get();
  if (moving.status() == mcx_cpp::StatusCode::OK) {
    clog << "Moved to point jpos_3" << endl;
  } else {
    helper::printStatus(moving);
    throw(runtime_error(string("Failed to moved to point jpos_3")));
  }

  robot.off();
}