/*
 * Developer : Alexey Zakharov (alexey.zakharov@vectioneer.com)
 * All rights reserved. Copyright (c) 2019 VECTIONEER.
 */

#include "helper.h"
#include "mcx-robot-control.h"

using namespace std;
using namespace mcx_robot_control;

int main(int argc, char* argv[]) {

  clog << "Motorcortex Robot Control library version: " << mcx_cpp::version() << endl;

  // setting url
  // url format: ws://address:req_port:sub_port
  // std::string url = "wss://192.168.2.100:5568:5567";
  std::string url = "wss://localhost:5568:5567";
  if (argc > 1) {
    url = std::string(argv[1]);
  }
  mcx_cpp::ConnectionOptions options{"mcx.cert.pem"};

  // Creating a parameter tree;
  mcx_cpp::ParameterTree parameter_tree;
  mcx_cpp::Request req{parameter_tree};
  mcx_cpp::Subscribe sub{req};
  helper::connect(url, options, parameter_tree, req, sub);

  // Subscribing to the log
  sub.subscribe({"root/logger/logOut"}, "log", 1).get().notify([](auto params) {
    std::string log;
    params[0].value(log);
    clog << "Log: " << log;
  });

  PoseTransformer pose_transformer{req};

  // Example of conversion between joint and cart coordinates
  auto ik_coords = pose_transformer.calcJointToCartPose(toRadians({0.0, 0.0, 90.0, 0.0, 90.0, 0.0})).get();
  if (ik_coords.status() == mcx_cpp::StatusCode::OK) {
    clog << "Transformed Joint to Cartesian Pose" << endl;
    clog << ik_coords << endl;
  } else {
    helper::printStatus(ik_coords);
    throw(runtime_error(string("Failed to transform Joint to Cartesian Pose")));
  }

  auto fk_coords = pose_transformer.calcCartToJointPose(ik_coords).get();
  if (fk_coords.status() == mcx_cpp::StatusCode::OK) {
    clog << "Transformed Cartesian Pose to Joint" << endl;
    clog << fk_coords << endl;
  } else {
    helper::printStatus(fk_coords);
    throw(runtime_error(string("Failed to transform Cartesian to Joint Pose")));
  }

  ik_coords = pose_transformer.calcJointToCartPose(fk_coords).get();
  if (ik_coords.status() == mcx_cpp::StatusCode::OK) {
    clog << "Transformed Joint to Cartesian Pose" << endl;
    clog << ik_coords << endl;
  } else {
    helper::printStatus(ik_coords);
    throw(runtime_error(string("Failed to transform Joint to Cartesian Pose")));
  }

  // Example of conversion between joint and cart coordinates
  fk_coords = pose_transformer.calcCartToJointPose({0.4145, 0, 0.334, 3.14159, 0, 3.14159}).get();
  if (fk_coords.status() == mcx_cpp::StatusCode::OK) {
    clog << "Transformed Cartesian Pose to Joint" << endl;
    clog << fk_coords << endl;
  } else {
    helper::printStatus(fk_coords);
    throw(runtime_error(string("Failed to transform Cartesian to Joint Pose")));
  }

  ik_coords = pose_transformer.calcJointToCartPose(fk_coords).get();
  if (ik_coords.status() == mcx_cpp::StatusCode::OK) {
    clog << "Transformed Joint to Cartesian Pose" << endl;
    clog << ik_coords << endl;
  } else {
    helper::printStatus(ik_coords);
    throw(runtime_error(string("Failed to transform Joint to Cartesian Pose")));
  }

  fk_coords = pose_transformer.calcCartToJointPose(ik_coords).get();
  if (fk_coords.status() == mcx_cpp::StatusCode::OK) {
    clog << "Transformed Cartesian Pose to Joint" << endl;
    clog << fk_coords << endl;
  } else {
    helper::printStatus(fk_coords);
    throw(runtime_error(string("Failed to transform Cartesian to Joint Pose")));
  }
}