/*
 * Developer : Alexey Zakharov (alexey.zakharov@vectioneer.com)
 * All rights reserved. Copyright (c) 2019 VECTIONEER.
 */

#include "helper.h"
#include "mcx-robot-control.h"

using namespace std;
using namespace mcx_robot_control;

constexpr double TIMEOUT_SEC = 5;

void off(const RobotCommand& robot) {
  auto off_done = robot.off(TIMEOUT_SEC).get();
  if (off_done.status() == mcx_cpp::StatusCode::OK) {
    clog << "System is in Off State" << endl;
  } else {
    helper::printStatus(off_done);
    throw(runtime_error(string("Failed to set Off State: ")));
  }
  //  this_thread::sleep_for(std::chrono::seconds(3));
}

void disengage(const RobotCommand& robot) {
  auto disengage_done = robot.disengage(TIMEOUT_SEC).get();
  if (disengage_done.status() == mcx_cpp::StatusCode::OK) {
    clog << "System is in Disengage State" << endl;
  } else {
    helper::printStatus(disengage_done);
    throw(runtime_error(string("Failed to set Disengage State: ")));
  }
  //  this_thread::sleep_for(std::chrono::seconds(3));
}

void engage(const RobotCommand& robot) {
  auto engage_done = robot.engage(TIMEOUT_SEC).get();
  if (engage_done.status() == mcx_cpp::StatusCode::OK) {
    clog << "System is in Engage State" << endl;
  } else {
    helper::printStatus(engage_done);
    throw(runtime_error(string("Failed to set Engage State: ")));
  }
  //  this_thread::sleep_for(std::chrono::seconds(3));
}

int main(int argc, char* argv[]) {

  clog << "Motorcortex Robot Control library version: " << mcx_cpp::version() << endl;

  // setting url
  // url format: ws://address:req_port:sub_port
  std::string url = "wss://localhost:5568:5567";
  //  std::string url = "wss://localhost:5568:5567";
  mcx_cpp::ConnectionOptions options{"mcx.cert.pem"};
  if (argc > 1) {
    url = std::string(argv[1]);
  }

  // Creating a parameter tree;
  mcx_cpp::ParameterTree parameter_tree;
  mcx_cpp::Request req{parameter_tree};
  mcx_cpp::Subscribe sub{req};
  helper::connect(url, options, parameter_tree, req, sub);

  // Subscribing to the log
  sub.subscribe({"root/logger/logOut"}, "log", 1).get().notify([](auto params) {
    std::string log;
    params[0].value(log);
    clog << "Log: " << log;
  });

  RobotCommand robot{req};

  // acknowledge
  robot.acknowledge();

  off(robot);
  engage(robot);
  off(robot);
  off(robot);
  disengage(robot);
  disengage(robot);
  robot.acknowledge();
  engage(robot);
  disengage(robot);
  engage(robot);
  off(robot);
  engage(robot);
  off(robot);
}